<?php

/**
 * GClient Class
 * 
 * Handles authentication and proxy settings
 * Uses singleton design pattern, so that not more than one client exists
 */
abstract class GClient {

	public $client;
	
	/**
	 * Create authenticated Google client
	 */
	protected function __construct ($service){
		// get login credentials
		$user = get_option ('gdocs_user');
		$pwd = get_option ('gdocs_pwd');
		
		// create HTTP client
		$httpClient = Zend_Gdata_ClientLogin::getHttpClient($user, $pwd, $service);
		
		$proxy_host = get_option ('gdocs_proxy_host');
		
		// if user specified a proxy
		if ($proxy_host){
			// set proxy
			$httpClient->setConfig (array (
				'adapter' => 'Zend_Http_Client_Adapter_Proxy',
				'proxy_host' => $proxy_host,
				'proxy_port' => get_option ('gdocs_proxy_port'),
				'proxy_user' => get_option ('gdocs_proxy_user'),
				'proxy_pass' => get_option ('gdocs_proxy_pwd')
			));
		}
		
		// create and return Google Docs client
		$this->client = new Zend_Gdata ($httpClient);
	}
	
	/**
	 * Get corresponding class to check if singleton already exists
	 */
	public static function getInstance ($service){
		switch ($service){
			case 'St':
				return GClient_St::getInstance ();
				break;
			case 'Doc':
				return GClient_Doc::getInstance ();
				break;
			default;
				return NULL;
		}
	}

}

/**
 *GClient for Google Documents
 */
class GClient_Doc extends GClient {

	private static $_singleton;

	protected function __construct (){
		parent::__construct (Zend_Gdata_Docs::AUTH_SERVICE_NAME);
	}
	
	public static function getInstance (){
		if (is_null (self::$_singleton)){
			// if object not created yet, construct it
			self::$_singleton = new GClient_Doc ();
		}
		return self::$_singleton->client;
	}

}

/**
 * GClient for Google Spreadsheets
 */
class GClient_St extends GClient {

	private static $_singleton;
	
	protected function __construct (){
		parent::__construct (Zend_Gdata_Spreadsheets::AUTH_SERVICE_NAME);
	}
	
	public static function getInstance (){
		if (is_null (self::$_singleton)){
			// if object not created yet, construct it
			self::$_singleton = new GClient_St ();
		}
		return self::$_singleton->client;
	}

	
}

?>
