<?php
/*
Plugin Name: Inline Google Docs
Description: Inline Google Docs allows the user to display contents of his google docs on his pages/posts, using shortcode for markup.
Version: 0.3.2
Author: Lim Jiunn Haur
Author URI: http://broken-watch.info/
*/

/*  Copyright 2008  Lim Jiunn Haur  (email : fisherman.jiang@broken-watch.info)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// load Zend library to use the Google Docs API
$path = dirname (__FILE__) . "/library";
set_include_path (get_include_path () . PATH_SEPARATOR . $path);

require_once 'Zend/Loader.php';
Zend_Loader::loadClass ('Zend_Http_Client');
Zend_Loader::loadClass ('Zend_Gdata');
Zend_Loader::loadClass ('Zend_Gdata_ClientLogin');
Zend_Loader::loadClass ('Zend_Gdata_Docs');
Zend_Loader::loadClass ('Zend_Gdata_Spreadsheets');
Zend_Loader::loadClass ('Zend_Gdata_Spreadsheets_DocumentQuery');

// load GClient class
require_once 'gclient.php';

/**
 * Shortcode handler
 */
function gdocs_display ($atts, $content = NULL){

	// check parameters
	if (is_null ($atts['type'])) return NULL;
	
	try {
	
		switch ($atts['type']){
		
		case 'document':
			
			// check parameters, cache
			if (is_null ($atts['id'])) return NULL;
			
			$fileName = dirname(__FILE__) . "/cache/" . gdocs_clean_filename ("doc_{$atts['id']}.html");
			if (gdocs_check_cache ($fileName, $html)) return $html;
			
			// get Http client
			$httpClient = gdocs_get_client ($atts);
			
			// connect to Google, get HTML
			$html = "<div class='gdocs' id='gdocs_{$atts['id']}'>" . $httpClient->request()->getBody() . "</div>";
			
			// keep a copy in cache
			if (get_option ('gdocs_cache_expiry')){
				$cacheFile = fopen ($fileName, 'wb');
				fwrite ($cacheFile, $html);
				fclose ($cacheFile);
			}
			
			// display
			return $html;
			
			break;
		
		case 'spreadsheet':
		
			// check parameters, cache
			if (is_null ($atts['wt_id']) || is_null ($atts['st_id'])) return NULL;
			
			$fileName = dirname(__FILE__) . "/cache/" . gdocs_clean_filename ("st_{$atts['st_id']}_{$atts['wt_id']}.html");
			if (gdocs_check_cache ($fileName, $html)) return $html;
		
			// get GData client
			$gsClient = GClient::getInstance('St');
			
			// get feed
			$feed = $gsClient->getFeed("http://spreadsheets.google.com/feeds/list/{$atts['st_id']}/{$atts['wt_id']}/private/full");
			if (!$feed) return NULL;
			
			// format
			$html = gdocs_get_html ($feed);
			
			// keep a copy in cache
			if (get_option ('gdocs_cache_expiry')){
				$cacheFile = fopen ($fileName, 'wb');
				fwrite ($cacheFile, $html);
				fclose ($cacheFile);
			}
			
			// display
			return $html;
			
			break;
			
		default:
			
			return NULL;
		
		}
		
	} catch (Zend_Http_Client_Adapter_Exception $e){
		// connnection problem, probably proxy
		return NULL;
	} catch (Zend_Gdata_App_AuthException $e) {
		// spreadsheets login problem
		return NULL;
	} catch (Exception $e) {
		// any other error
		return NULL;
	}
	
}

/**
 * Check if the doc/spreadsheet exists in cache
 */
function gdocs_check_cache ($name, &$html){

	// if HIT, return true, set $html
	if (file_exists ($name) && (time () - get_option ('gdocs_cache_expiry') * 60 < filemtime ($name))) {
	
		$html = file_get_contents ($name);
		if ($html) {			 
			 return true;
		}
	
	}
	
	// if MISS, return false
	return false;

}

/**
 * Function to clean a filename string so it is a valid filename
 */
function gdocs_clean_filename ($filename){
	$reserved = preg_quote('\/:*?"<>|', '/');//characters that are  illegal on any of the 3 major OS's
	//replaces all characters up through space and all past ~ along with the above reserved characters
	return preg_replace("/([\\x7f-\\xff{$reserved}])/e", "_", $filename);
}

/**
 * Hook for options page
 */
function gdocs_options (){

	if (function_exists ('add_options_page')){
		// add an options page that is printed by gdocs_options_setup
		add_options_page ('Inline Google Docs', 'G Docs', 'manage_options', basename(__FILE__), 'gdocs_options_setup');
		
	} 

}

/**
 * Setup and print options page
 */
function gdocs_options_setup (){

?>
	<div class='wrap'>
		<h2>Inline Google Docs</h2>
		<form method='post' action='options.php'>
			<?php wp_nonce_field ('update-options'); ?>
			<!-- Begin G Docs options -->
			<?php gdocs_print_login (); ?>
			<?php gdocs_print_proxy (); ?>
			<?php gdocs_print_cache (); ?>
			
<?php

	try {
		
		// get Google Docs client, login
		$gdClient = GClient::getInstance('Doc');
		if (is_null ($gdClient)) throw new Exception ();
		
		gdocs_print_docs ($gdClient);
		
		// get Google Spreasheets client
		$gsClient = GClient::getInstance('St');
		if (is_null ($gsClient)) throw new Exception ();
		
		gdocs_print_sts ($gdClient, $gsClient);
		
	} catch (Zend_Http_Client_Adapter_Exception $e){
		// connnection problem ?>
		<div class='error' id='message' style='background-color: rgb(255, 170, 150);'><p><strong><?php _e("A connection error has occurred. Are you behind a proxy?");  ?></strong></p></div>
	<?php } catch (Zend_Gdata_App_AuthException $e) {
		// google docs login problem
		if (!get_option ('gdocs_user') || !get_option ('gdocs_pwd')){ ?>
			<div class='error' id='message' style='background-color: rgb(255, 170, 150);'><p><strong><?php _e("Please enter your username and password."); ?></strong></p></div>
		<?php }else { ?>
			<div class='error' id='message' style='background-color: rgb(255, 170, 150);'><p><strong><?php _e("The plugin was unable to login to the Google service. Did you give us the wrong password/username?") ?></strong></p></div>
		<?php }
	} catch (Exception $e){ ?>
		<div class='error' id='message' style='background-color: rgb(255, 170, 150);'><p><strong><?php _e("A connection error has occurred. Perhaps you should have some coffee?") ?></strong></p></div>
	<?php } ?>			
			<p class='submit'>
				<input type='submit' name='Submit' value="<?php _e('Save changes') ?>" />
			</p>
			
			<input type="hidden" name="action" value="update" />
			<input type="hidden" name="page_options" value="gdocs_user,gdocs_pwd,gdocs_proxy_host,gdocs_proxy_port,gdocs_proxy_user,gdocs_proxy_pwd, gdocs_cache_expiry" />
			
			
		</form>

	</div>

<?php

}

/**
 * Prints login credentials form
 */
function gdocs_print_login (){
?>

			<!-- Login Credentials -->
			<h3>Google Account Login</h3>
			<table cellpadding="2" cellspacing="5">
				<tbody>
					<tr valign="top">
						<th scope="row">Username:</th>
						<td><input type="text" size="40" name="gdocs_user" value="<?php echo get_option ('gdocs_user'); ?>" /></td>
					</tr>
					<tr valign="top">
						<th scope="row">Password:</th>
						<td><input type="password" size="40" name="gdocs_pwd" value="<?php echo get_option ('gdocs_pwd'); ?>" /></td>
					</tr>
				</tbody>
			</table>

<?php
}

/**
 * Prints proxy settings form
 */
function gdocs_print_proxy (){
?>

			<!-- Proxy Settings -->
			<h3>Proxy Settings</h3>
			<table cellpadding="2" cellspacing="5">
				<tbody>
					<tr valign="top">
						<th scope="row">Host:</th>
						<td><input type="text" size"40" name="gdocs_proxy_host" value="<?php echo get_option ('gdocs_proxy_host'); ?>" /></td>
					</tr>
					<tr valign="top">
						<th scope="row">Port:</th>
						<td><input type="text" size"40" name="gdocs_proxy_port" value="<?php echo get_option ('gdocs_proxy_port'); ?>" /></td>
					</tr>
					<tr valign="top">
						<th scope="row">Username:</th>
						<td><input type="text" size"40" name="gdocs_proxy_user" value="<?php echo get_option ('gdocs_proxy_user'); ?>" /></td>
					</tr>
					<tr valign="top">
						<th scope="row">Password:</th>
						<td><input type="password" size"40" name="gdocs_proxy_pwd" value="<?php echo get_option ('gdocs_proxy_pwd'); ?>" /></td>
					</tr>
				</tbody>
			</table>

<?php
}

/**
 * Prints cache settings form
 */
function gdocs_print_cache (){
?>

			<!-- Cache Settings -->
			<h3>Cache Settings</h3>
			<small>Set to <span style="color: blue">'0'</span> to turn caching off.</small>
			<table cellpadding="2" cellspacing="5">
				<tbody>
					<tr valign="top">
						<th scope="row">Lifespan:</th>
						<td><input type="text" size"40" name="gdocs_cache_expiry" value="<?php echo get_option ('gdocs_cache_expiry'); ?>" /> minutes</td>
					</tr>
				</tbody>
			</table>

<?php
}

/**
 * Prints list of documents
 */
function gdocs_print_docs ($gdClient) {

	// get documents feed
	$feed = $gdClient->getFeed ("http://docs.google.com/feeds/documents/private/full/-/document");

?>		
			<!-- Document List -->
			<h3>Google Documents</h3>
			<small>Use the corresponding Document ID in your shortcode.<br />
			<span style="color: blue">[gdocs id=<em>doc_id</em> type='document']</span>.</small>
			<table cellpadding="2" cellspacing="5">
				<thead style="text-align:left">
					<tr><th>Title</th><th>Document ID</th></tr>
				</thead>
				<tbody>

<?php 
	// check if feed has been retrieved
	if (!$feed) throw new Exception ();
	
	foreach ($feed->entries as $entry){
?>
					<tr><td><?php echo $entry->getTitle(); ?></td><td><?php
					
					// extract id
					preg_match ('/^.+%3A(.+)$/', $entry->getId()->getText(), $matches);
					echo $matches[1];
					
?></td></tr>

<?php 
	} 
?>
				</tbody>
			</table>
			
<?php

}

/**
 * Prints list of spreadsheets
 */
function gdocs_print_sts ($gdClient, $gsClient){

	// get spreadsheets feed
	$feed = $gdClient->getFeed ("http://docs.google.com/feeds/documents/private/full/-/spreadsheet");
	
?>
		<!-- Spreadsheet List -->
		<h3>Google Spreadsheets</h3>
		<small>Use the corresponding Spreadsheet ID and Worksheet ID in your shortcode.<br />
		<span style="color: blue">[gdocs st_id=<em>spreadsheet_id</em> wt_id=<em>worksheet_id</em> type='spreadsheet']</span>.</small>

		<table cellpadding="2" cellspacing="5">
			<thead style="text-align:left">
				<tr><th>Title</th><th>Spreadsheet ID</th><th>Worksheets</th></tr>
			</thead>
			<tbody>

<?php
	// check if feed has been retrieved
	if (!$feed) throw new Exception ();
	
	foreach ($feed->entries as $entry){ 
?>

				<tr><td><?php echo $entry->getTitle(); ?></td><td><?php 
				
				// extract spreadsheet id
				preg_match ('/^.+%3A(.+)$/', $entry->getId()->getText(), $matches);
				echo $matches[1];
				
				?></td><td><?php

				// get worksheets feed
				$wtFeed = $gsClient->getFeed ("http://spreadsheets.google.com/feeds/worksheets/{$matches[1]}/private/full");
				
				if ($wtFeed) foreach ($wtFeed->entries as $entry){
				
					// extract worksheet id
					$wtId = split ('/', $entry->getId()->getText());
					echo $wtId[8] . "&nbsp;";
					
				}

?></td></tr>
<?php 
	}
?>
			</tbody>
		</table>
		
<?php

}

/**
 * Format HTML to display spreadsheet content in a table
 */
function gdocs_get_html ($feed){
	
	// convert to spreadsheet list feed object
	$feed = new Zend_Gdata_Spreadsheets_ListFeed ($feed->getDom ());

	// start table
	$ar = split ('/', $feed->getId()->getText());
	$html = "<table class='gdocs' id='gdocs_" . $ar[6] . "'>\r\n";
	$html .= "\t<thead>\r\n";
	
	// extract column headings
	$firstRow = $feed->entries[0]->getCustom ();
	$html .= "\t\t<tr id='row_0'>\r\n\t\t\t";
	$k = 0;
	foreach ($firstRow as $colHead){
		$html .= "<th class='col_{$k}'>" . $colHead->getColumnName () . "</th>";
	}
	$html .= "\r\n\t\t</tr>\r\n";
	
	$html .= "\t</thead>\r\n";
	$html .= "\t<tbody>\r\n";

	// for every row
	$i = 1;
	foreach ($feed->entries as $entry){
	
		// start table row
		$html .= "\t\t<tr id='row_{$i}'>\r\n\t\t\t";
		
		// get all the cells in this row
		$cells = $entry->getCustom ();
		
		// for every cell, display the contents
		$j = 0;
		foreach ($cells as $cell){
			$html .= "<td class='col_{$j}'>" . $cell->getText () . "</td>";
			$j++;
		}
		
		// end table row
		$html .= "\r\n\t\t</tr>\r\n";
		$i++;
		
	}
	
	$html .= "\t</tbody>\r\n";		
	$html .= "</table>\r\n";
	
	return $html;

}

/**
 * Get Zend Http Client to access raw Google Docs content
 */
function gdocs_get_client ($atts){

	// Zend HTTP Client singleton
	static $httpClient;
	
	// if it does not exist, construct a new one
	is_null ($httpClient) ? $httpClient = gdocs_create_client () : NULL;
	
	// set parameters
	$httpClient->setParameterGet (array (
		action => 'fetch',
		justBody => 'true',
		revision =>'_latest',
		editMode => 'false',
		docID => $atts['id']
	));
	
	return $httpClient;

}

/**
 * Create Zend Http Client (only if it does not already exist)
 */
function gdocs_create_client (){

	$httpClient = new Zend_Http_Client ('http://docs.google.com/RawDocContents');
		
	$proxy_host = get_option ('gdocs_proxy_host');
	
	// if user specified a proxy
	if ($proxy_host){
		// set proxy
		$httpClient->setConfig (array (
			'adapter' => 'Zend_Http_Client_Adapter_Proxy',
			'proxy_host' => $proxy_host,
			'proxy_port' => get_option ('gdocs_proxy_port'),
			'proxy_user' => get_option ('gdocs_proxy_user'),
			'proxy_pass' => get_option ('gdocs_proxy_pwd')
		));
	}
	
	return $httpClient;

}

/**
 * Install plugin
 */
function gdocs_install (){
	// login credentials
	add_option ('gdocs_user');
	add_option ('gdocs_pwd');
	
	// proxy config
	add_option ('gdocs_proxy_host');
	add_option ('gdocs_proxy_port');
	add_option ('gdocs_proxy_user');
	add_option ('gdocs_proxy_pwd');
	
	add_option ('gdocs_cache_expiry');
}

/**
 * Uninstall plugin
 */
function gdocs_uninstall (){
	delete_option ('gdocs_user');
	delete_option ('gdocs_pwd');
	delete_option ('gdocs_proxy_host');
	delete_option ('gdocs_proxy_port');
	delete_option ('gdocs_proxy_user');
	delete_option ('gdocs_proxy_pwd');
	delete_option ('gdocs_cache_expiry');
}

// add hooks
add_action ('admin_menu', 'gdocs_options');
add_shortcode ('gdocs', 'gdocs_display');
register_activation_hook (__FILE__, 'gdocs_install');
register_deactivation_hook (__FILE__, 'gdocs_uninstall');

?>